/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define WIDTH 800
#define HEIGHT 600

#include <SDL.h>
#include <SDL_image.h>

#define IS_CENTER(cx, cy, x, y) \
	((x + WIDTH/2 == cx) && (y + HEIGHT/2 == cy))

SDL_Rect points[] = {
  {400, 300, 800, 600},
  {450, 350, 800, 600}
};

SDL_Rect *
GetNextPoint (void)
{
  static SDL_Rect rect = {0, 0, WIDTH, HEIGHT};
  static int cur = 0;
  int dx, dy;
  int next;
  next = (cur + 1) % (sizeof (points) / sizeof (SDL_Rect));
  if (IS_CENTER (points[next].x, points[next].y, rect.x, rect.y))
  {
    cur = next;
    next = (cur + 1) % (sizeof (points) / sizeof (SDL_Rect));
  }
  dx = (points[next].x > points[cur].x) ? 1 : -1;
  dy = (points[next].y > points[cur].y) ? 1 : -1;
  rect.x = (rect.x + dx) % WIDTH;
  rect.y = (rect.y + dy) % HEIGHT;
  return &rect;
}

void
ShowPoint (SDL_Surface *screen, SDL_Surface *image, SDL_Rect *rect)
{
  SDL_BlitSurface (image, rect, screen, NULL);
  SDL_UpdateRect (screen, 0, 0, 0, 0);
}

Uint32
ShowNext (Uint32 interval, void *data)
{
  SDL_UserEvent event;
  event.type = SDL_USEREVENT;
  event.code = 0;
  SDL_PushEvent ((SDL_Event *) &event);
  return 33;
}

int
main (int argc, char **argv)
{
  SDL_Surface *screen;
  SDL_Surface *image;
  SDL_Event event;
  SDL_Init (SDL_INIT_VIDEO | SDL_INIT_TIMER);
  screen = SDL_SetVideoMode (800, 600, 32, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
  image = IMG_Load ("/home/cascardo/fotos/debconf.jpg");
  SDL_AddTimer (0, ShowNext, NULL);
  while (SDL_WaitEvent (&event))
  {
    if (event.type == SDL_KEYDOWN)
      break;
    else if (event.type == SDL_USEREVENT)
      ShowPoint (screen, image, GetNextPoint ());
  }
  SDL_FreeSurface (image);
  SDL_Quit ();
  return 0;
}
